<?php

namespace Mcnd\CLI\Lib;

class AsciiFormatter
{
    public array $stack = [];

    protected string $latest;

    public function parse(array|string $content) : array|string
    {
        return is_array($content) ? array_map([ $this, 'parseContent' ], $content) : $this->parseContent($content);
    }

    protected function parseContent(string $text) : string
    {
        if (str_contains($text, '#[')) {
            # Opening 'tag'
            $text = preg_replace_callback("#(\#\[)(.*?)(\])#s", function ($matches) {
                if ( empty($stack) ) {
                    $this->stack[] = $this->translateFormat("c:default, bg:default, f:normal");
                }
                else {
                    $this->stack[] = $this->latest;
                }

                $this->latest = $this->translateFormat($matches[2]);

                return $this->latest;
            }, $text);

            # Closing 'tag'
            $text = preg_replace_callback("#(\[\#\])#s", function ($matches) {
                $close = $this->stack ? array_pop($this->stack) : "";

                return $close;
            }, $text);
        }

        return $text;
    }

    protected function translateFormat(string $source) : string
    {
        $valueType =
        $color = $background = $format = null;

        foreach(array_map(fn($e) => trim(strtolower($e)), explode(',', $source)) as $formatting) {
            if ( ! strstr($formatting, ':' )) {
                throw new \InvalidArgumentException(
                    sprintf("Arguments must be called with their variable's name (%s)", $formatting)
                );
            }

            list($scope, $value) = explode(':', $formatting, 2);

            switch($scope) {
                case 'color':
                case 'c':
                    $type = ValueTypeEnum::autodetect($value);

                    if ($type === ValueTypeEnum::Ascii16) {
                        $color = ColorEnum::mapFrom($value);
                    }
                    elseif ($type === ValueTypeEnum::Ascii256) {
                        $color = RgbColor::mapFrom($value);
                    }

                    break;

                case 'background':
                case 'bg':
                case 'b':
                    $type = ValueTypeEnum::autodetect($value);

                    if ($type === ValueTypeEnum::Ascii16) {
                        $background = BackgroundColorEnum::mapFrom($value);
                    }
                    elseif ($type === ValueTypeEnum::Ascii256) {
                        $background = RgbBackground::mapFrom($value);
                    }

                break;

                case 'format':
                case 'f':
                    $format = FormatEnum::mapFrom($value);
                break;
            }
        }

        return ($color ? $color->output() : "") . ($background ? $background->output() : "") . ($format ? $format->output() : "");
    }
}